@@ -4,8 +4,8 @@ from sqlalchemy import event, MetaData
 from sqlalchemy.engine import Engine
 
 from logic_bank.rule_bank.rule_bank import RuleBank
-from logic_bank.exec_trans_logic.listeners import before_flush, before_commit
 from logic_bank.rule_bank import rule_bank_withdraw
+from logic_bank.exec_trans_logic.listeners import before_flush, before_commit
 from sqlalchemy.orm import session
 
 
@@ -22,7 +22,7 @@ def setup(a_session: session):
 
     rules_bank.orm_objects = {}
     rules_bank._at = datetime.now()
-    return
+    return rules_bank
 
 def setup_early_row_event_all_classes(early_row_event_all_classes: callable):
     ll = RuleBank()
